<?php

class Op5reports_options extends Report_options
{
	public static $type = false;
	protected $subs = array();

	public function __construct($options=false)
	{
		$metaprops = array('report_id' => $this->properties['report_id'], 'report_name' => $this->properties['report_name']);
		$this->properties = array_merge($metaprops, array(
			'use_pnp' => array('type' => 'bool', 'default' => false),
			'use_summary' => array('type' => 'bool', 'default' => false),
		));
		foreach ($metaprops as $optname => $_) {
			if (isset($options[$optname]))
				unset($options[$optname]);
		}
		if (is_a($options, 'Op5reports_options')) {
			parent::__construct($options);
			foreach ($options->subs as $sub) {
				$class = get_class($sub);
				$this->add_sub(new $class($sub));
			}
		} else {
			if (static::$type == 'avail')
				$class = 'Avail_options';
			else
				$class = 'Sla_options';
			parent::__construct($options);
			$this->add_sub($class::create_options_obj($options));
			$this->add_sub(Summary_options::create_options_obj($options));
		}
	}

	public static function discover_options($input = false)
	{
		if (static::$type == 'avail')
			$class = 'Avail_options';
		else
			$class = 'Sla_options';
		return $class::discover_options($input);
	}

	/**
	 * @param $type string
	 * @param $options array|Report_options
	 */
	private function add_sub(Report_options $sub)
	{
		$this->options = array_merge($sub->options, $this->options);
		$this->subs[] = $sub;
	}

	public function set($name, $value)
	{
		$res = parent::set($name, $value);
		foreach ($this->subs as $sub) {
			if (($res += $sub->set($name, $value))) {
				# handle weird side-effects
				$this->options = array_merge($this->options, $sub->options);
			}
		}
		return (bool)$res;
	}

	public function get_alternatives($key)
	{
		$out = parent::get_alternatives($key);
		if ($out !== false)
			return $out;
		$current = false;
		foreach ($this->subs as $sub) {
			$out = $sub->get_alternatives($key);
			if ($out !== false) {
				if($out) {
					// return the first non-falsy value
					return $out;
				}
				$current = $out; // might for example be an empty array or a '0'
			}
		}
		return $current;
	}

	public function offsetGet($str)
	{
		$out = parent::offsetGet($str);
		if ($out !== NULL)
			return $out;
		$current = NULL;
		foreach ($this->subs as $sub) {
			$out = $sub->offsetGet($str);
			if ($out !== NULL)
				return $out;
		}
	}

	public function offsetExists($key)
	{
		// The underlying `offsetExists` uses the raw `properties` property,
		// and not the the `properties` function that we overload to extend
		// the available properties.
		return isset($this->properties()[$key]);
	}

	public function get_value($key)
	{
		$out = parent::get_value($key);
		if ($out !== false)
			return $out;
		$current = false;
		foreach ($this->subs as $sub) {
			$out = $sub->get_value($key);
			if ($out !== false) {
				if($out) {
					// return the first non-falsy value
					return $out;
				}
				$current = $out; // might for example be an empty array or a '0'
			}
		}
		return $current;
	}

	function next() {
		while (true) {
			$x = next($this->options);
			if ($x === false)
				break;
			if (isset($this->properties[key($this->options)]) && !isset($this->properties[key($this->options)]['generated']))
				break;
			foreach ($this->subs as $sub) {
				if (isset($sub->properties[key($this->options)]) && !isset($sub->properties[key($this->options)]['generated']))
					break 2;
			}
		}
		return $x;
	}

	public function expand() {
		foreach ($this->properties as $key => $_) {
			$this[$key] = $this[$key];
		}
		foreach ($this->subs as $sub) {
			$sub->expand();
			$this->options = array_merge($sub->options, $this->options);
		}
	}

	public function properties()
	{
		$props = $this->properties;
		foreach ($this->subs as $sub)
			$props = array_merge($props, $sub->properties());
		return $props;
	}
}
